using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Data.SqlClient;
using System.Collections;

// The namespace is a way for us to classify classes into to groups, all of the classes that we create in this project
// are assigned to a "ProductList" namespace. If we want to use this class in a form we need to import the
// "ProductList" namespace.
namespace ProductList
{

    // This class is documented with XML tags, we use the <remarks> tags to describe a class or a region and use
    // the <summary> tags to describe specific properties and methods. The content of a summary tag will be visible as
    // a description for the property or method when someone are looking at it in user code. XML comments can be
    // collapsed by the minus sign and expanded by the plus sign.

    /// <summary>
    /// This class is used to insert, update and select headcategories in an MS SQL database table and to
    /// create objects of one headcategory. This class offers flexibility to insert, update, and select headcategories
    /// by using several overloaded constructors, properties and several overloaded methods.  
    /// </summary>
    public class HeadCategories
    {

        #region Variables

        /// <remarks>
        /// The variables in this class are declared as private and thereby only reachable by the properties and
        /// methods inside this class. These private variables is used to hold information about one headcategory.
        /// When we set values for these variables and when we get values from these we variables we are always using
        /// the properties in this class even when refereing to these variables inside this class. If we had declared
        /// these variables as protected they would also be reachable by classes that inherits from this class but we
        /// dont want that.  
        /// </remarks>

        private Int32 _HeadId;
        private string _LinkName;
        private string _Description;

        #endregion

        #region Constructors

        /// <remarks>
        /// This class has several overloaded constructors that can be used to when someone creates a new instance of
        /// this class. We dont have to write the "Overloads" for constructors that should be overloaded, each 
        /// overloaded constructor must have a unique order, number of or type of parameters. The "this" keyword is
        /// used to be specific that we are referencing a property or method in this class. We are always using
        /// the properties to set values for private variables in the constructors.  
        /// </remarks>

        public HeadCategories()
        {
            this.HeadId = 0;
            this.LinkName = "Need to update this";
            this.Description = "Need to update this";
        }

        public HeadCategories(string pLinkName, string pDescription)
        {
            this.LinkName = pLinkName;
            this.Description = pDescription;
        }

        public HeadCategories(Int32 pHeadId, string pLinkName, string pDescription)
        {
            this.HeadId = pHeadId;
            this.LinkName = pLinkName;
            this.Description = pDescription;
        }

        protected HeadCategories(SqlDataReader reader)
        {
            this.HeadId = Convert.ToInt32(reader["HeadId"]);
            this.LinkName = reader["LinkName"].ToString();
            this.Description = reader["Description"].ToString();
        }

        #endregion

        #region Properties

        /// <remarks>
        /// Public properties that are used to set and get values to/from private variables.
        /// </remarks>

        /// <summary>
        /// Is used to set or get a head id regarding one headcategory.
        /// </summary>
        public Int32 HeadId
        {
            get { return _HeadId; }
            set { _HeadId = value; }
        }

        /// <summary>
        /// Is used to set or get a link name regarding one headcategory.
        /// </summary>
        public string LinkName
        {
            get { return _LinkName; }
            set { _LinkName = value; }
        }

        /// <summary>
        /// Is used to set or get a description regarding one headcategory.
        /// </summary>
        public string Description
        {
            get { return _Description; }
            set { _Description = value; }
        }

        #endregion

        #region Methods

        /// <remarks>
        /// Public methods that are used to perform tasks regarding head categories.
        /// </remarks>

        /// <summary>
        /// Insert one headcategory row with LinkName and Description based on the values for the 
        /// variables LinkName and Description. You need to set values for these properties before calling
        /// this method, HeadId is a identity column that increments with 1.
        /// </summary>
        /// <example>This example shows how to use this method with an empty constructor
        /// <code>
        /// // Declare InsertRow as a new instance of the HeadCategories class, InsertRow is a new HeadCategories object
        /// HeadCategories InsertRow = new HeadCategories();
        /// // Set values for properties regarding the InsertRow object, LinkName and Description
        /// InsertRow.LinkName = txtLinkName.Text;
        /// InsertRow.Description = txtDescription.Text;
        /// // Add error handling to catch exceptions thrown by the method.
        /// try
        /// {
        ///    // Call the method to insert one row of headcategories in a database table
        ///    InsertRow.InsertOneRow();
        /// }
        /// catch (Exception ex)
        /// {
        ///    Response.Write(ex.Message);
        /// }
        /// </code>
        /// </example>
        /// <example2>This example shows how to use this method when parameters are supplied to the constructor
        /// <code>
        /// // Declare InsertRow as a new instance of the HeadCategories class and send two parameters
        /// // to the constructor.
        /// HeadCategories InsertRow = new HeadCategories(txtLinkName.Text, txtDescription.Text);
        /// // Add error handling to catch exceptions thrown by the method.
        /// try
        /// {
        ///    // Call the method to insert one row of headcategories in a database table
        ///    InsertRow.InsertOneRow();
        /// }
        /// catch (Exception ex)
        /// {
        ///    Response.Write(ex.Message);
        /// }
        /// </code>
        /// </example2>
        public void InsertOneRow()
        {
            
            // Declare variables for a connection string and a INSERT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "INSERT INTO HeadCategories (LinkName, Description) VALUES (@LinkName, @Description)";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Add parameters.
                cmd.Parameters.AddWithValue("@LinkName", this.LinkName);
                cmd.Parameters.AddWithValue("@Description", this.Description);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the INSERT statement.
                    cmd.ExecuteNonQuery();

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Insert one headcategory row by using the parameter values supplied when this method is called, 
        /// the HeadId column is a identity that increments with 1.
        /// </summary>
        /// <param name="pLinkName">A short name for links in a menu (varchar, 50)</param>
        /// <param name="pDescription">A 200 character description of the headcategory</param>
        /// <example>This example shows how to use this method
        /// <code>
        /// // Declare InsertRow as a new instance of the HeadCategories class
        /// HeadCategories InsertRow = new HeadCategories();
        /// // Add error handling to catch exceptions thrown by the method.
        /// try
        /// {
        ///     // Call a method with two parameters to insert one headcategory row
        ///     InsertRow.InsertOneRow(txtLinkName.Text, txtDescription.Text);
        /// }
        /// catch (Exception ex)
        /// {
        ///     Response.Write(ex.Message);
        /// }
        /// </code>
        /// </example>
        public void InsertOneRow(string pLinkName, string pDescription)
        {
            // Declare variables for a connection string and a INSERT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "INSERT INTO HeadCategories (LinkName, Description) VALUES (@LinkName, @Description)";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Add parameters.
                cmd.Parameters.AddWithValue("@LinkName", pLinkName);
                cmd.Parameters.AddWithValue("@Description", pDescription);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the INSERT statement
                    cmd.ExecuteNonQuery();

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Insert many headcategory rows with a SqlDataAdapter and a DataTable, create a datatable that contains
        /// two columns (LinkName and Description).
        /// </summary>
        /// <param name="pHeadCategoryDT">A DataTable with two columns (LinkName(50) and Desription(200))</param>
        public void InsertManyRows(DataTable pHeadCategoryDT)
        {
            // Declare variables for a connection string and a INSERT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "INSERT INTO HeadCategories (LinkName, Description) VALUES (@LinkName, @Description)";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataAdapter.
                SqlDataAdapter ad = new SqlDataAdapter();

                // Add parameters.
                cmd.Parameters.Add("@LinkName", SqlDbType.VarChar, 50, "LinkName");
                cmd.Parameters.Add("@Description", SqlDbType.VarChar, 200, "Description");

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Set the insert command for the SqlDataAdapter.
                    ad.InsertCommand = cmd;

                    // Excecute the INSERT statement.
                    ad.Update(pHeadCategoryDT);

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    cmd.Dispose();
                    ad.Dispose();
                }
            }
        }

        /// <summary>
        /// Insert many headcategory rows with SqlBulkCopy and a DataTable, create a datatable that contains
        /// two columns (LinkName and Description).
        /// </summary>
        /// <param name="pHeadCategoryDT">A DataTable contain two columns (LinkName(50) and Desription(200))</param>
        public void InsertSqlBulk(DataTable pHeadCategoryDT)
        {
            // Declare a variable for a connection string.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a new SqlBulkCopy.
                SqlBulkCopy bc = new SqlBulkCopy(cn);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // The HeadCategories table has 3 columns but the pHeadCategoryDT DataTable have only 
                    // two columns and we therefore have to add column mappings to match the DataTable with
                    // the MS SQL database table.
                    bc.ColumnMappings.Add("LinkName", "LinkName");
                    bc.ColumnMappings.Add("Description", "Description");

                    // Set the name of the destination table in the database
                    bc.DestinationTableName = "HeadCategories";

                    // Write from the DataTable to the destination table
                    bc.WriteToServer(pHeadCategoryDT);

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    bc.Close();
                }
            }
        }

        /// <summary>
        /// Update one headcategory row where the HeadId equals the HeadId variable in this class, you need to set values
        /// for HeadId, LinkName and Description with a constructor or with properties before using this method.
        /// </summary>
        public void UpdateOneRow()
        {
            // Declare variables for a connection string and a UPDATE statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "UPDATE HeadCategories SET LinkName = @LinkName, Description = @Description WHERE HeadId = @Id";
   
            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Add parameters.
                cmd.Parameters.AddWithValue("@Id", this.HeadId);
                cmd.Parameters.AddWithValue("@LinkName", this.LinkName);
                cmd.Parameters.AddWithValue("@Description", this.Description);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Excecute the UPDATE statement
                    cmd.ExecuteNonQuery();

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Update one headcategory row where the HeadId equals the supplied parameter pId, you need to add 
        /// three parameters to use this method.
        /// </summary>
        /// <param name="pId">Id number for a headcategory</param>
        /// <param name="pLinkName">A short name for links in a menu (varchar, 50)</param>
        /// <param name="pDescription">A 200 character despription of the headcategory</param>
        public void UpdateOneRow(Int32 pId, string pLinkName, string pDescription)
        {
            // Declare variables for a connection string and a UPDATE statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "UPDATE HeadCategories SET LinkName = @LinkName, Description = @Description WHERE HeadId = @Id";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Add parameters.
                cmd.Parameters.AddWithValue("@Id", pId);
                cmd.Parameters.AddWithValue("@LinkName", pLinkName);
                cmd.Parameters.AddWithValue("@Description", pDescription);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Excecute the UPDATE statement
                    cmd.ExecuteNonQuery();

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Select one headcategory row where the HeadId equals the value for the variable HeadId in this class, which you
        /// have to set with a constructor or a property before using this method. The selected row is used to set the
        /// properties for HeadId, LinkName and Description so you have to get values from the properties to get the
        /// selected data.
        /// </summary>
        public void SelectOneRow()
        {
            // Declare variables for a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM HeadCategories WHERE HeadId = @HeadId";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats why 
                // we dont write the "New" keyword.
                SqlDataReader reader = null;

                // Add parameters.
                cmd.Parameters.AddWithValue("@HeadId", this.HeadId);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data.
                    reader = cmd.ExecuteReader(CommandBehavior.SingleRow);

                    // Loop the reader.
                    while (reader.Read())
                    {
                        this.HeadId = Convert.ToInt32(reader["HeadId"]);
                        this.LinkName = reader["LinkName"].ToString();
                        this.Description = reader["Description"].ToString();
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }
        }

        /// <summary>
        /// Select one headcategory row where HeadId equals pID, you need supply a parameter value for HeadId to use
        /// this method. The selected row is used to set values for the properties: HeadId, LinkName and Description.
        /// You need to get the values from these properties after calling this method.
        /// </summary>
        /// <param name="pId">Id number for a headcategory</param>
        public void SelectOneRow(Int32 pID)
        {
            // Declare variables for a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM HeadCategories WHERE HeadId = @HeadId";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats 
                // why we dont write the "new" keyword.
                SqlDataReader reader = null;

                // Add parameters.
                cmd.Parameters.AddWithValue("@HeadId", pID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data.
                    reader = cmd.ExecuteReader(CommandBehavior.SingleRow);

                    // Loop the reader.
                    while (reader.Read())
                    {
                        this.HeadId = Convert.ToInt32(reader["HeadId"]);
                        this.LinkName = reader["LinkName"].ToString();
                        this.Description = reader["Description"].ToString();
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }
        }

        /// <summary>
        /// Select one headcategory row where HeadId equals pId, you have to supply a value for the parameter pId to use
        /// this method. The selected row is returned as an array of 3 strings (0 to 2).
        /// </summary>
        /// <param name="pId">Id number for a headcategory</param>
        /// <returns>A one-dimensional 0-based array with 3 strings (HeadId, LinkName, Description)</returns>
        public string[] SelectOneRowToArray(Int32 pId)
        {

            // Declare a OneRowArray as a StringArray with 3 values (0 to 2)
            string[] OneRowArray = new string[3];

            // Declare a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM HeadCategories WHERE HeadId = @HeadId";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats why 
                // we dont write the "New" keyword.
                SqlDataReader reader = null;

                // Add parameters.
                cmd.Parameters.AddWithValue("@HeadId", pId);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data. 
                    reader = cmd.ExecuteReader(CommandBehavior.SingleRow);

                    // Loop the reader.
                    while (reader.Read())
                    {
                        // Fill the StringArray with values
                        OneRowArray[0] = reader["HeadId"].ToString();
                        OneRowArray[1] = reader["LinkName"].ToString();
                        OneRowArray[2] = reader["Description"].ToString();
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }

            // Return a one-dimensional string array
            return OneRowArray;
        }

        /// <summary>
        /// Select one headcategory row where HeadId are equal to the supplied value for the parameter pId. This function
        /// returns an array list with one headcategory object that is created with a protected constructor that accepts
        /// a SqlDataReader as a parameter.
        /// </summary>
        /// <param name="pId">Id number for a headcategory</param>
        /// <returns>A ArrayList with one headcategory object (HeadId, LinkName, Description)</returns>    
        public ArrayList SelectOneRowToArrayList(Int32 pId)
        {
            // Declare OneRowToArrayList as a new ArrayList
            ArrayList OneRowToArrayList = new ArrayList();

            // Declare variables for a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM HeadCategories WHERE HeadId = @HeadId";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats why 
                // we dont write the "New" keyword.
                SqlDataReader reader = null;

                // Add parameters.
                cmd.Parameters.AddWithValue("@HeadId", pId);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data.
                    reader = cmd.ExecuteReader(CommandBehavior.SingleRow);

                    // Loop the reader.
                    while (reader.Read())
                    {
                        // Add a headcategory object to the ArrayList.
                        OneRowToArrayList.Add(new HeadCategories(reader));
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }

            // Return a ArrayList
            return OneRowToArrayList;
        }

        /// <summary>
        /// Select all headcategory rows to an 2-dimensional Array of strings.
        /// </summary>
        /// <returns>An 2-dimensional Array of strings with three columns (HeadId, LinkName, Description)</returns>
        public string[,] SelectAllTo2DArray()
        {
            // Create a two-dimensional string array.
            string[,] AllTo2DArray;

            // Declare RowNumber as a Integer that will be used to fill the 2D array with values and TotalNumberOfRows
            // as a Integer to be able to determine the size of the 2D array.
            Int32 RowNumber = default(Int32);
            Int32 TotalNumberOfRows = default(Int32);

            // Declare variables for a connection string and two SELECT statements.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT Count(HeadId) As Rows FROM HeadCategories";
            string sql2 = "SELECT * FROM HeadCategories";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create two SqlCommands.
                SqlCommand cmd = new SqlCommand(sql, cn);
                SqlCommand cmd2 = new SqlCommand(sql2, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats why 
                // we dont write the "New" keyword.
                SqlDataReader reader = null;

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Get the number of rows - 1
                    TotalNumberOfRows = Convert.ToInt32(cmd.ExecuteScalar()) - 1;

                    // Declare AllTo2DArray with rows and columns
                    AllTo2DArray = new string[TotalNumberOfRows + 1, 3];

                    // Execute the second SELECT statement.
                    reader = cmd2.ExecuteReader();

                    // Loop the reader.
                    while (reader.Read())
                    {
                        // Fill the 2D StringArray with values
                        AllTo2DArray[RowNumber, 0] = reader["HeadId"].ToString();
                        AllTo2DArray[RowNumber, 1] = reader["LinkName"].ToString();
                        AllTo2DArray[RowNumber, 2] = reader["Description"].ToString();

                        // Add 1 to RowNumber
                        RowNumber = RowNumber + 1;
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    cmd.Dispose();
                    cmd2.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }

            // Return a 2-dimensional string array
            return AllTo2DArray;
        }

        /// <summary>
        /// Select all headcategory rows to an ArrayList of headcategory objects (HeadId, LinkName, Description). An 
        /// ArrayList of objects can be used as a datasource for a repeater control.
        /// </summary>
        /// <returns>An ArrayList of headcategory objects, each object consists of (HeadId, LinkName, Description)</returns>
        public ArrayList SelectAllToArrayList()
        {
            // Declare AllToArrayList as a new ArrayList
            ArrayList AllToArrayList = new ArrayList();

            // Declare a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM HeadCategories";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats why we 
                // dont write the "New" keyword.
                SqlDataReader reader = null;

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data.
                    reader = cmd.ExecuteReader();

                    // Loop the reader.
                    while (reader.Read())
                    {
                        // Add a headcategory object to the ArrayList
                        AllToArrayList.Add(new HeadCategories(reader));
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }

            // Return a ArrayList
            return AllToArrayList;
        }

        /// <summary>
        /// Select all headcategory rows to a DataTable with three columns (HeadId, LinkName, Description). A DataTable
        /// can be used as a datasource for a repeater control.
        /// </summary>
        /// <returns>A DataTable three columns (HeadId, LinkName, Description) that contains headcategory rows</returns>
        public DataTable SelectAllToDataTable()
        {

            // Declare AllHeadCategories as a new DataTable
            DataTable AllHeadCategories = new DataTable();

            // Declare a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM HeadCategories";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataAdapter.
                SqlDataAdapter ad = new SqlDataAdapter();

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Set the select command for the SqlDataAdapter
                    ad.SelectCommand = cmd;

                    // Use the SqlDataAdapter to insert data in a DataTable
                    ad.Fill(AllHeadCategories);
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of resources to avoid memory leakage.
                    cmd.Dispose();
                    ad.Dispose();
                }
            }

            // Return a DataTable
            return AllHeadCategories;
        }

        #endregion
    }
}

